/*******************************************************************
 GOUKEI.EXE                              (c)2012, ST.(JES M.Sai)
     金額の合計算出
--------------------------------------------------------------------
 (1) Ubuntu 12.04LTS
 2012.10.18  V1.01U Ubuntuへ移植

*******************************************************************/
/*******************************************************************
 GOUKEI.EXE                              (c)1994,99, ST.(JES M.Sai)
     金額の合計算出
--------------------------------------------------------------------
 (1) EPSON PC-386NAR   (80386SX-25MHz) + EPSON MS-DOS Ver.3.00
 LSI C-86 Compiler ver 3.30c  [Aug 19 1993]
      lcc -j GOUKEI.C
 (2) EPSON PC-486NAS   (80486SX-25MHz) + NEC   MS-DOS Ver.5.00A
 LSI C-86 Compiler ver 3.30c  [Aug 19 1993]
      lcc -j GOUKEI.C
 (3) HP 200LX (80186-7.91HHz) + MS-DOS ROM 5.00 + OSW 日本語化キット 1.10 
 LSI C-86 Compiler ver 3.30c  [Dec 17 1993]
      lcc -j GOUKEI.C

   MAKEFILE: なし
--------------------------------------------------------------------
 1994.01.09  V0.00  制作開始
 1994.02.11  V1.00  完成
 1994.11.30  V1.01  BUG FIX 漢字コードの２バイト目の読み飛ばし忘れ

*******************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <jctype.h>
#include <ctype.h>


/*********************       定　　　数       *********************/

#define TITLE			"\nGOUKEI V1.03U 2012,ST.\n"
#define LINE_MAX		256


/*********************       変　　　数       *********************/


/*********************       関　　　数       *********************/

/*==================================================================
名　称：使用法表示

形　式：int lookup_usage(void)

引　数：なし

戻り値：-1:固定値

説　明：○プログラムの使用法を表示します。
==================================================================*/
int lookup_usage(void)
{
	fputs(TITLE, stderr);
	fputs("金額の合計算出\n", stderr);
	fputs("　テキスト中から「\\」を探して合計値を出力します。\n\n", stderr);
	fputs("　ただし「\\」は１行に１個のみに対応します。\n\n", stderr);
	fputs("使用法 :  GOUKEI fname\n\n", stderr);
	fputs("fname:読込むテキストファイル名\n", stderr);
	return (-1);
}


/*==================================================================
名　称：sjis漢字コードの検査

形　式：

引　数：

戻り値：

説　明：○sjisの第１バイトの検査を行う。
        V1.02U 追加
==================================================================*/
int iskanji(int c)
{
	c &= 0x00FF;
	if ((c >= 0x81) && (c <= 0x9F)) {
		return -1;
	} else if ((c >= 0xE0) && (c <= 0xFC)) {
		return -1;
	}
	return 0;
}

/*==================================================================
名　称：文字列を数値化して加算

形　式：int addnum(char *pin_buf, long *sum)

引　数：

戻り値：

説　明：○
==================================================================*/
int addnum(char *pin_buf, long *sum)
{
	char num_buf[LINE_MAX];
	long num;
	int pos;

	pos = 0;
	if ((*pin_buf == '-') || (*pin_buf == '+')) {	/* V1.03U +/-とも収入とする */
		pin_buf++;
		num_buf[pos++] = '0';
	} else {
		num_buf[pos++] = '-';
	}

	while (*pin_buf) {
		if (isdigit(*pin_buf)) {		/* 数字 */
			num_buf[pos++] = *pin_buf++;
		} else if (*pin_buf == ',') {	/* 読み飛ばし */
			pin_buf++;
		} else {
			break;
		}
	}
	num_buf[pos] = '\0';
	num = strtol(num_buf, NULL, 10);
	*sum += num;
	//printf("%ld\r\n", num);
	return (0);
}


/*==================================================================
名　称：

形　式：

引　数：

戻り値：

説　明：○
==================================================================*/
int getsum(char *fname, long *sum_g)
{
	long sum;
	FILE *fp;
	char in_buf[LINE_MAX];
	char *p;

	fp = fopen(fname, "r");
	if (fp == NULL) {
		printf("OPEN ERROR %s\n", fname);
		return (-1);
	}
	sum = 0;
	while (fgets(in_buf, LINE_MAX, fp) != NULL) {
		p = in_buf;
		while (*p) {
			if (iskanji(*p)) {
				p++;
				p++;		/* V1.01 漢字コードの２バイト目の読み飛ばし */
			} else if (*p == '\\') {
				p++;
				addnum(p, &sum);
				break;
			} else {
				p++;
			}
		}
	}
	*sum_g += sum;
	fclose(fp);			/* V1.02 */
	return (0);
}


/*==================================================================
名　称：

形　式：

引　数：

戻り値：

説　明：○
==================================================================*/
int putnum(long sum)
{
	char str[LINE_MAX];
	int pos;
	long a;

	putchar('\\');
	if (sum < 0) {
		sum *= -1;
		putchar('-');
	} else {
		putchar('+');
	}

	pos = 0;
	while (1) {
		a = sum / 10;
		str[pos++] = '0' + (sum - a*10);
		sum = a;
		if (sum <= 0) break;
		if (pos % 4 == 3) {
			str[pos++] = ',';
		}
	}
	str[pos] = '\0';
	pos = strlen(str);
	if (str[0] == ',') {
		pos--;
	}
	while (pos--) {
		putchar(str[pos]);
	}
	putchar('\n');
	return (0);
}


/*==================================================================
名　称：

書　式：

引　数：コマンドライン文字列
		argv[1]:
			path:変更するドライブ・ディレクトリ
			-r  :ドライブ・ディレクトリ復帰

戻り値： 0:正常終了
		-1:異常終了

説　明：○
		　
		○
		　
==================================================================*/
int main(int argc, char *argv[])
{
	long sum;
	int i;

	if (argc >= 2) {
		sum = 0;
		for (i=1; argv[i]!='\0'; i++) {
			getsum(argv[i], &sum);
		}
		putnum(sum);
	} else {
		return(lookup_usage());
	}
	return (0);
}


/*******************************************************************
 1994.01.09  V0.00  制作開始
 1994.02.11  V1.00  完成
 1994.11.30  V1.01  BUG FIX
                    漢字コードの２バイト目の読み飛ばし忘れを修正
 1996.06.13  V1.02  BUG FIX
                    FCLOSEのつけ忘れを修正
 1999.03.01  V1.03  収入の入力に対応
 Ubuntu 対応
 2012.10.12  V1.03U Ubuntu対応
                    対象ファイルはShift-JISのみ(オリジナルのまま)
                    プログラムのソースファイルはUTF-8に変換
*******************************************************************/

